@@ -75,7 +75,7 @@ struct hardhat_maker {
 	/* Indicates what went wrong in case of failure */
 	char *error;
 	/* The superblock, as it will be created at the end */
-	struct hardhat_superblock superblock;
+	struct hardhat superblock;
 };
 
 /* The only case in which it is impossible to allocate the
@@ -113,12 +113,15 @@ export bool hardhat_maker_fatal(hardhat_maker_t *hhm) {
 	- remove occurrences of .
 	- resolve occurrences of ..
 */
-__attribute__((optimize(99)))
-export size_t hardhat_normalize(uint8_t *dst, const uint8_t *src, size_t size) {
-	uint8_t *cur, *end;
-	const uint8_t *sep, *nul;
+__attribute__((optimize(3)))
+export size_t hardhat_normalize(void *to, const void *from, size_t size) {
+	uint8_t *dst, *cur, *end;
+	const uint8_t *src, *sep, *nul;
 	size_t len;
 
+	dst = to;
+	src = from;
+
 	nul = src + size;
 	cur = dst;
 	do {
@@ -132,8 +135,7 @@ export size_t hardhat_normalize(uint8_t *dst, const uint8_t *src, size_t size) {
 			// do nothing
 		} else if(len == 2 && src[0] == '.' && src[1] == '.') {
 			end = memrchr(dst, '/', (size_t)(cur - dst));
-			if(end)
-				cur = end;
+			cur = end ? end : dst;
 		} else {
 			if(cur > dst)
 				*cur++ = '/';
@@ -162,7 +164,7 @@ export size_t hardhat_normalize(uint8_t *dst, const uint8_t *src, size_t size) {
 	x/a/2
 	x/b/1
 */
-__attribute__((optimize(99)))
+__attribute__((optimize(3)))
 export int hardhat_cmp(const void *a, size_t al, const void *b, size_t bl) {
 	const uint8_t *as, *bs, *ap, *bp;
 	uint8_t ac = 0, bc = 0;
@@ -261,18 +263,35 @@ static void hhm_set_error(hardhat_maker_t *hhm, const char *fmt, ...) {
 }
 
 static uint32_t makeseed(void) {
-	struct timespec ts[5];
-	int clocks = 2;
-	clock_gettime(CLOCK_REALTIME, ts);
-	clock_gettime(CLOCK_MONOTONIC, ts + 1);
-#ifdef CLOCK_MONOTONIC_RAW
-	clock_gettime(CLOCK_MONOTONIC_RAW, ts + clocks++);
-#endif
+	struct timespec ts[8];
+	int clocks = 0;
+	if(!clock_gettime(CLOCK_REALTIME, ts + clocks))
+		clocks++;
+	if(!clock_gettime(CLOCK_MONOTONIC, ts + clocks))
+		clocks++;
 #ifdef CLOCK_PROCESS_CPUTIME_ID
-	clock_gettime(CLOCK_PROCESS_CPUTIME_ID, ts + clocks++);
+	if(!clock_gettime(CLOCK_PROCESS_CPUTIME_ID, ts + clocks))
+		clocks++;
 #endif
 #ifdef CLOCK_THREAD_CPUTIME_ID
-	clock_gettime(CLOCK_THREAD_CPUTIME_ID, ts + clocks++);
+	if(!clock_gettime(CLOCK_THREAD_CPUTIME_ID, ts + clocks))
+		clocks++;
+#endif
+#ifdef CLOCK_MONOTONIC_RAW
+	if(!clock_gettime(CLOCK_MONOTONIC_RAW, ts + clocks))
+		clocks++;
+#endif
+#ifdef CLOCK_REALTIME_COARSE
+	if(!clock_gettime(CLOCK_REALTIME_COARSE, ts + clocks))
+		clocks++;
+#endif
+#ifdef CLOCK_MONOTONIC_COARSE
+	if(!clock_gettime(CLOCK_MONOTONIC_COARSE, ts + clocks))
+		clocks++;
+#endif
+#ifdef CLOCK_BOOTTIME
+	if(!clock_gettime(CLOCK_BOOTTIME, ts + clocks))
+		clocks++;
 #endif
 	return calchash_murmur3((const void *)ts, sizeof *ts * clocks, getpid());
 }
@@ -434,7 +453,10 @@ export bool hardhat_maker_add(hardhat_maker_t *hhm, const void *key, uint16_t ke
 		value = ht->buf[hp].data;
 		if(value == EMPTYHASH) {
 			if(!addhash(ht, hash, hhm->recnum)) {
-				hhm->error = enomem;
+				if(hhm->error != enomem) {
+					free(hhm->error);
+					hhm->error = enomem;
+				}
 				hhm->failed = true;
 				return false;
 			}
@@ -478,7 +500,10 @@ export bool hardhat_maker_add(hardhat_maker_t *hhm, const void *key, uint16_t ke
 		hhm->recbufsize *= 2;
 		buf = realloc(hhm->recbuf, hhm->recbufsize * sizeof *hhm->recbuf);
 		if(!buf) {
-			hhm->error = enomem;
+			if(hhm->error != enomem) {
+				free(hhm->error);
+				hhm->error = enomem;
+			}
 			hhm->failed = true;
 			return false;
 		}
@@ -574,7 +599,7 @@ static int qsort_hash_cmp(const void *a, const void *b) {
 }
 
 /* Find the longest common prefix (on ‘/’ boundaries) */
-__attribute__((optimize(99)))
+__attribute__((optimize(3)))
 static size_t common_parents(const uint8_t *a, size_t al, const uint8_t *b, size_t bl) {
 	size_t cl = 0, l, i;
 	uint8_t ac, bc;
@@ -694,8 +719,10 @@ export bool hardhat_maker_finish(hardhat_maker_t *hhm) {
 				he = realloc(ht->buf, ht->size * sizeof *ht->buf);
 				if(!he) {
 					hhm->failed = true;
-					free(hhm->error);
-					hhm->error = enomem;
+					if(hhm->error != enomem) {
+						free(hhm->error);
+						hhm->error = enomem;
+					}
 					return false;
 				}
 				ht->buf = he;
